#include "Extruded.h"

class Cylinder : public Extruded
{
public:
  Cylinder(int samples,float r,float h) : Extruded(samples,48) {
    height = h;
    radius = r;
    init();
  }

protected:
  virtual glm::vec3 normal(int i,int j) {
    return this->base(i);
  }

  virtual glm::ivec3 name(int i,int j) {
    return glm::ivec3(0,1,2);
  }

  virtual glm::vec3 weight(int i,int j) {
    float t = float(j)/48;
    float cutOne = 1.0f / 3;
    float cutTwo = 2.0f / 3;
    float margin = 1.0f / 24;
    if(t <= cutOne - margin)
	    return glm::vec3(1.0,0.0,0.0);
    if(t <= cutOne + margin)
	    return glm::vec3(2.5 - 6*t,-1.5 + 6*t,0.0);
    if(t <= cutTwo - margin)
      return glm::vec3(0.0,1.0,0.0);
    if(t <= cutTwo + margin)
      return glm::vec3(0.0,4.5 - 6*t,-3.5 + 6*t);
    return glm::vec3(0.0,0.0,1.0);
  }
  
  virtual glm::vec3 base(int i) {
    float s = ((float) i)/numSamples;
    return radius*glm::vec3(cos(-s * 2.0f * 3.14159f), 0, sin(-s * 2.0f * 3.14159f));
  }

  virtual glm::mat4 T(float t) {
    return glm::translate(glm::mat4(1.0),glm::vec3(0,height*t,0));
  }
private:
  float height;
  float radius;
};
