#include <cmath>
#include <vector>
#include <iostream>
#include <glm/gtc/matrix_transform.hpp>
#include "Extruded.h"

using namespace std;

Extruded::Extruded(int samples,int slices) {
	numSamples = samples;
	numSlices = slices;
	numVertices = numSamples * (numSlices + 1);
	numIndices = numSamples * numSlices * 6;
}

void Extruded::init() {
	for (int i = 0; i < numVertices; i++) { vertices.push_back(glm::vec3()); }
	for (int i = 0; i < numVertices; i++) { normals.push_back(glm::vec3()); }
	for (int i = 0; i < numVertices; i++) { names.push_back(glm::ivec3()); }
	for (int i = 0; i < numVertices; i++) { weights.push_back(glm::vec3()); }
	for (int i = 0; i < numIndices; i++) { indices.push_back(0); }

	// calculate the base vertices
	for (int i = 0; i < numSamples; i++) {
		vertices[i] = this->base(i);
		normals[i] = this->normal(i,0);
		names[i] = this->name(i,0);
		weights[i] = this->weight(i,0);
	}

	// Calculate the remaining vertices
	for (int j = 1; j <= numSlices; j++) {
		float t = ((float) j)/numSlices;
		glm::mat4 tMat = this->T(t);
		for (int i = 0; i < numSamples; i++) {
			vertices[j*numSamples+i] = glm::vec3(tMat * glm::vec4(vertices[i],1.0f));
			normals[j*numSamples+i] = normal(i,j);
			names[j*numSamples+i] = name(i,j);
			weights[j*numSamples+i] = weight(i,j);
		}
	}

	// calculate triangle indices
	for (int slice = 0; slice < numSlices; slice++) {
		for (int i = 0; i < numSamples-1; i++) {
			indices[((slice*numSamples + i) * 2) * 3 + 0] = slice*numSamples + i;
			indices[((slice*numSamples + i) * 2) * 3 + 1] = slice*numSamples + i + 1;
			indices[((slice*numSamples + i) * 2) * 3 + 2] = (slice+1)*numSamples + i;
			indices[((slice*numSamples + i) * 2 + 1) * 3 + 0] = slice*numSamples + i + 1;
			indices[((slice*numSamples + i) * 2 + 1) * 3 + 1] = (slice + 1)*numSamples + i + 1;
			indices[((slice*numSamples + i) * 2 + 1) * 3 + 2] = (slice + 1)*numSamples + i;
		}
		indices[((slice*numSamples + numSamples - 1) * 2) * 3 + 0] = slice*numSamples + numSamples - 1;
		indices[((slice*numSamples + numSamples - 1) * 2) * 3 + 1] = slice*numSamples;
		indices[((slice*numSamples + numSamples - 1) * 2) * 3 + 2] = (slice+1)*numSamples + numSamples - 1;
		indices[((slice*numSamples + numSamples - 1) * 2 + 1) * 3 + 0] = slice*numSamples;
		indices[((slice*numSamples + numSamples - 1) * 2 + 1) * 3 + 1] = (slice + 1)*numSamples;
		indices[((slice*numSamples + numSamples - 1) * 2 + 1) * 3 + 2] = (slice + 1)*numSamples + numSamples - 1;
	}
}

int Extruded::getNumVertices() { return numVertices; }
int Extruded::getNumIndices() { return numIndices; }
std::vector<int> Extruded::getIndices() { return indices; }
std::vector<glm::vec3> Extruded::getVertices() { return vertices; }
std::vector<glm::vec3> Extruded::getNormals() { return normals; }
std::vector<glm::ivec3> Extruded::getBoneNames() { return names; }
std::vector<glm::vec3> Extruded::getBoneWeights() { return weights; }
