#ifndef RIGTFORM_H
#define RIGTFORM_H

#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp> // glm::translate, glm::rotate, glm::scale, glm::perspective

class RigTForm {
  glm::vec3 t_; // translation component
  glm::mat4 r_;  // rotation component

public:
  RigTForm() : t_(0.0f), r_(1.0f) {}

  RigTForm(const glm::vec3& t, const glm::mat4& r): t_(t),r_(r) {}

  explicit RigTForm(const glm::vec3& t) : t_(t), r_(1.0f) {}

  explicit RigTForm(const glm::mat4& r) : t_(0.0f), r_(r) {}

  glm::vec3 getTranslation() const {
    return t_;
  }

  glm::mat4 getRotation() const {
    return r_;
  }

  RigTForm& setTranslation(const glm::vec3& t) {
    t_ = t;
    return *this;
  }

  RigTForm& setRotation(const glm::mat4& r) {
    r_ = r;
    return *this;
  }

  glm::mat4 toMatrix() const {
    return glm::translate(glm::mat4(1.0f), t_)*r_;
  }
};

#endif
