#include "Skeleton.h"
#include <glm/gtc/matrix_transform.hpp> // glm::translate, glm::rotate, glm::scale, glm::perspective

Bone::Bone(Bone* parent, const glm::vec3& offset, const RigTForm& transform)
	: parent_(parent), offset_(offset), transform_(transform)
{
}

void Bone::rotateBy(const glm::mat4& rotation)
{
	transform_.setRotation(transform_.getRotation() * rotation);
}

void Bone::setRotation(const glm::mat4& rotation)
{
	transform_.setRotation(rotation);
}

glm::mat4 Bone::getModelMatrix() const
{
	if (parent_ != NULL)
		return parent_->getModelMatrix() * transform_.toMatrix();
	return  transform_.toMatrix();
}

glm::mat4 Bone::getBoneMatrix() const
{
	return this->getModelMatrix()*glm::translate(glm::mat4(1.0f),offset_);
}

Skeleton::Skeleton()
{
}

Skeleton::~Skeleton()
{
	std::map<int, Bone*>::iterator iter;
    for (iter = bones_.begin(); iter != bones_.end(); iter++)
		delete iter->second;
	bones_.clear();
}

Bone* Skeleton::addBone(int name,Bone* parent,const glm::vec3& offset,const RigTForm& transform)
{
	Bone* newBone = new Bone(parent,offset,transform);
	bones_[name] = newBone;
	return newBone;
}

Bone* Skeleton::getNamedBone(int index)
{
	return bones_[index];
}
