#version 430

layout (location = 0) in vec3 vertPos;
layout (location = 1) in vec3 vertNormal;
layout (location = 2) in ivec3 vertNames;
layout (location = 3) in vec3 vertWeights;

out vec3 varyingNormal;
out vec3 varyingLightDir;
out vec3 varyingVertPos;

struct PositionalLight
{	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	vec3 position;
};
struct Material
{	vec4 ambient;
	vec4 diffuse;
	vec4 specular;
	float shininess;
};

uniform vec4 globalAmbient;
uniform PositionalLight light;
uniform Material material;
uniform mat4 proj_matrix;
uniform mat4 bones[8];
uniform mat4 boneNormals[8];

void main(void)
{	
  varyingNormal = vec3((vertWeights.x*boneNormals[vertNames.x]+
                    vertWeights.y*boneNormals[vertNames.y]+
                    vertWeights.z*boneNormals[vertNames.z]) * vec4(vertNormal, 0.0));
  
  varyingVertPos = ((vertWeights.x*bones[vertNames.x]+
                 vertWeights.y*bones[vertNames.y]+
                 vertWeights.z*bones[vertNames.z]) * vec4(vertPos, 1.0)).xyz;
 
  varyingLightDir = light.position - varyingVertPos;
	
  gl_Position = proj_matrix * vec4(varyingVertPos,1.0);
}
